'''
Created: Aug 4, 2013
Last modified: Aug 14, 2013
@author Flaise Saffron
'''

import pygame
import math
import sys
import random
import os


######################################################################## engine

### Not used in this app anymore but still a useful library function
def get_stick(joy, axisX, axisY):
	state = ( joy.get_axis(axisX), joy.get_axis(axisY) )
	
	magSq = magnitudeSq2d(state)
	
	### The MadCatz controller reports a vector magnitude of >1 when the sticks
	### are tilted towards the corners, which seems to be because of a dead
	### zone around the outside. Normalizing the vector whenever its magnitude
	### is >1 seems to fix this.
	if magSq > 1:
		return normalize2d(state)
	
	dead_zone_radius = .2
	
	### Circular dead zone
	if magSq < dead_zone_radius ** 2:
		return (0, 0)
	
	### Scale usable input space so it goes from 0->1 instead of abruptly
	### jumping to the hardware's representation of the edge of the dead zone.
	mag = math.sqrt(magSq)
	max_mag = 1 - dead_zone_radius
	new_mag = (mag - dead_zone_radius) / max_mag
	
	return mag_scale_to_2d(state, new_mag)

### Tampering with the pygame Vector classes to add the necessary functionality
### didn't seem worth it; these few functions are all that are needed anyway
def sum2d(a, b):
	return (a[0] + b[0], a[1] + b[1])

def dif2d(a, b):
	return (a[0] - b[0], a[1] - b[1])

def sum3d(a, b):
	return (a[0] + b[0], a[1] + b[1], a[2] + b[2])

def magnitudeSq2d(vec):
	return vec[0] ** 2 + vec[1] ** 2

def magnitude2d(vec):
	return math.sqrt(magnitudeSq2d(vec))

def normalize2d(vec):
	return mag_scale_2d(vec, 1 / magnitude2d(vec))

def int2d(vec):
	return (int(vec[0]), int(vec[1]))

def mag_scale_2d(vec, scalar):
	return (vec[0] * scalar, vec[1] * scalar)

def mag_scale_to_2d(vec, scalar):
	if not math.isfinite(scalar):
		raise Exception() ### because this was probably not intended
	return mag_scale_2d(vec, scalar / magnitude2d(vec))
	### Just let it throw when the vector has a magnitude of zero; there's no
	### real way to klodge that into a usable number because it would require
	### a direction.
	### A real programming language would allow this function to have
	### preconditions.

def flipY2d(vec):
	return (vec[0], -vec[1])

def flip2d(vec):
	return (-vec[0], -vec[1])

def scale2d(vec, scalar):
	return (vec[0] * scalar, vec[1] * scalar)

def basic_product_2d(a, b):
	return (a[0] * b[0], a[1] * b[1])

def negate2d(vec):
	return (-vec[0], -vec[1])

def negate3d(vec):
	return (-vec[0], -vec[1], -vec[2])


EVENT_DRAW = 100

event_handlers = [] ### contains: (predicate:function, callback:function)
next_event_handlers = []
def proc_event(event):
	global event_handlers, next_event_handlers
	
	event_handlers = next_event_handlers
	next_event_handlers = list(event_handlers)
	for handler in event_handlers:
		if handler[0](event):
			handler[1](event)
			
def proc_events():
	for event in pygame.event.get():
		proc_event(event)
		
def event_loop():
	while True:
		proc_events()
		proc_event(pygame.event.Event(EVENT_DRAW))
		pygame.display.flip()

def on_event(predicate, callback):
	### TODO: take priorty as parameter so screen flip and hud drawing can be
	### registered in any order
	
	### TODO: what about attaching events to other events?
	
	removed = False
	def invoke(event):
		if removed: return
		
		callback(event)
	
	def remove(): ### TODO: Use linked list
		nonlocal removed
		if removed: return
		removed = True
		
		next_event_handlers.remove(handler)
	
	handler = (predicate, invoke)
	next_event_handlers.append(handler)
	return remove

def on_event_once(predicate, callback):
	stop = on_event(predicate, lambda a: (stop(), callback(a)))


def render_text(text, position, anchor, dest_surface, font, color_fill, color_outline):
	surface = font.render(text, True, color_outline)
	position = sum2d(position, negate2d(basic_product_2d(surface.get_size(), anchor)))
	
	dest_surface.blit(surface, sum2d(position, (2, 0)))
	dest_surface.blit(surface, sum2d(position, (-2, 0)))
	dest_surface.blit(surface, sum2d(position, (0, 2)))
	dest_surface.blit(surface, sum2d(position, (0, -2)))
	
	surface = font.render(text, True, color_fill)
	dest_surface.blit(surface, position)
	
	return pygame.Rect(position, surface.get_size())



########################################################################### app

pygame.init()
surface_screen = pygame.display.set_mode((1024, 768))
pygame.display.set_caption('Squish the Bugs v0.1.2 by Flaise Saffron')

joysticks = [pygame.joystick.Joystick(x) for x in range(pygame.joystick.get_count())]
for joy in joysticks:
	joy.init()

color_BG = pygame.Color(220, 220, 255)
color_HUD = pygame.Color(255, 240, 240)
color_HUD_outline = pygame.Color(60, 30, 30)

class AppEvent:
	(
		INPUT_DIRECTION, ### kwargs: direction
		BLOCK_MOVED, ### kwargs: block
		BUG_SQUISHED, ### kwargs: block
		RESTART ### void
	) = range(1000, 1004)
	
class Direction:
	NORTH, EAST, SOUTH, WEST = range(4)

on_event(
	lambda a: a.type == pygame.KEYDOWN and a.key == pygame.K_UP,
	lambda a: proc_event(pygame.event.Event(AppEvent.INPUT_DIRECTION, direction=Direction.NORTH))
)
on_event(
	lambda a: a.type == pygame.KEYDOWN and a.key == pygame.K_DOWN,
	lambda a: proc_event(pygame.event.Event(AppEvent.INPUT_DIRECTION, direction=Direction.SOUTH))
)
on_event(
	lambda a: a.type == pygame.KEYDOWN and a.key == pygame.K_RIGHT,
	lambda a: proc_event(pygame.event.Event(AppEvent.INPUT_DIRECTION, direction=Direction.EAST))
)
on_event(
	lambda a: a.type == pygame.KEYDOWN and a.key == pygame.K_LEFT,
	lambda a: proc_event(pygame.event.Event(AppEvent.INPUT_DIRECTION, direction=Direction.WEST))
)

previous_hat = (0, 0)
def hat_to_direction4(event):
	global previous_hat
	if event.value[0] != previous_hat[0]:
		if event.value[0] == 1:
			proc_event(pygame.event.Event(AppEvent.INPUT_DIRECTION, direction=Direction.EAST))
		elif event.value[0] == -1:
			proc_event(pygame.event.Event(AppEvent.INPUT_DIRECTION, direction=Direction.WEST))
	if event.value[1] != previous_hat[1]:
		if event.value[1] == 1:
			proc_event(pygame.event.Event(AppEvent.INPUT_DIRECTION, direction=Direction.NORTH))
		elif event.value[1] == -1:
			proc_event(pygame.event.Event(AppEvent.INPUT_DIRECTION, direction=Direction.SOUTH))
	previous_hat = event.value
on_event(lambda a: a.type == pygame.JOYHATMOTION, lambda a: hat_to_direction4(a))


on_event(lambda a: a.type == EVENT_DRAW, lambda a: surface_screen.fill(color_BG))
on_event(lambda a: a.type == pygame.QUIT, lambda a: sys.exit())
on_event(lambda a: a.type == pygame.KEYDOWN and a.key == pygame.K_ESCAPE, lambda a: sys.exit())


camera = ((0, 0, 0), 7) # ((x, y, z), view_height) ~~~ width depends on screen

def move_camera_to(dest):
	if len(dest) != 3: raise Exception() 
	global camera
	camera = (dest, camera[1])


TILE_W = 1
TILE_L = .8
TILE_H = .4

class Tile:
	def __init__(self, surface, reg_position, reg_width, is_filler, relative_layer, is_hud):
		self.surface = surface
		self.reg_position = reg_position
		self.reg_width = reg_width
		self.is_filler = is_filler
		self.relative_layer = relative_layer
		self.is_hud = is_hud

image_directory = 'images'
def load_tile(file_name, reg_position, reg_width, is_filler=False, relative_layer=0, is_hud=False):
	return Tile(
		pygame.image.load(os.path.join(image_directory, file_name)).convert_alpha(),
		reg_position, reg_width, is_filler, relative_layer, is_hud
	)

tile_gray = load_tile('Plain Block.png', (50, 90), 100, True, 0, False)
tile_dirt = load_tile('Dirt Block.png', (50, 90), 100, True, 0, False)
tile_grass = load_tile('Grass Block.png', (50, 90), 100, True, 0, False)
tile_water = load_tile('Water Block.png', (50, 90), 100, True, -1, False)
tile_rock = load_tile('Rock.png', (50, 90), 100, False, 0, False)
tile_tree = load_tile('Tree Tall.png', (50, 90), 100, False, 0, False)
tile_player = load_tile('Character Cat Girl.png', (50, 90), 100, False, 0, False)
tile_player_in_water = load_tile('Character Cat Girl In Water.png', (50, 90), 100, False, 0, False)
tile_bug = load_tile('Enemy Bug.png', (50, 90), 100, False, 0, False)
tile_bug_dead = load_tile('Enemy Bug Squished.png', (50, 90), 100, False, -1, False)
tile_speech = load_tile('SpeechBubble.png', (-35, 190), 100, False, 0, True)
tile_wood = load_tile('Wood Block.png', (50, 90), 100, True, 0, False)
tile_question = load_tile('Question.png', (-35, 190), 100, False, 1, True) ### Uses font Palatino Linotype
tile_bush_0 = load_tile('Tree Short.png', (50, 90), 100, False, 0, False)
tile_bush_1 = load_tile('Tree Ugly.png', (50, 90), 100, False, 0, False)

shadow_e = load_tile('Shadow East.png', (50, 90), 100)
shadow_w = load_tile('Shadow West.png', (50, 90), 100)
shadow_s = load_tile('Shadow South.png', (50, 90), 100)
shadow_n = load_tile('Shadow North.png', (50, 90), 100)
shadow_side_w = load_tile('Shadow Side West.png', (50, 90), 100)
shadow_se = load_tile('Shadow South East.png', (50, 90), 100)
shadow_nw = load_tile('Shadow North West.png', (50, 90), 100)
shadow_sw = load_tile('Shadow South West.png', (50, 90), 100)
shadow_ne = load_tile('Shadow North East.png', (50, 90), 100)



blocks = []
position_to_blocklist = {} ### contains: {(x, y, z): [blocks]}

class BlockType:
	PLAYER, BUG, ROCK, TREE, FILLER, BUG_DEAD, SPEECH, WATER = range(8)
class Block:
	def __init__(self, position, tile, block_type):
		self.position = position
		self.tile = tile
		self.block_type = block_type
		
		blocks.append(self)
		update_blocks()
		
	def move(self, delta):
		self.move_to(sum3d(self.position, delta))
	
	def move_to(self, dest):
		self.position = dest
		update_blocks()
		proc_event(pygame.event.Event(AppEvent.BLOCK_MOVED, block=self, dest=dest))
	
	def remove(self):
		position_to_blocklist[self.position].remove(self)
		if len(position_to_blocklist[self.position]) == 0:
			del position_to_blocklist[self.position]
		blocks.remove(self)
		proc_event(pygame.event.Event(AppEvent.BLOCK_MOVED, block=self, dest=None))
		


def clear_blocks():
	global blocks
	blocks = [] ### Not sure why List doesn't have a clear method...
	
	position_to_blocklist.clear()
on_event(lambda a: a.type == AppEvent.RESTART, lambda a: clear_blocks())

def update_blocks():
	blocks.sort(key=lambda a: a.position[2] * 10000 + a.position[1] + a.tile.relative_layer + (100000000 if a.tile.is_hud else 0))
	position_to_blocklist.clear()
	for block in blocks:
		position = block.position
		if position not in position_to_blocklist:
			position_to_blocklist[position] = []
		position_to_blocklist[position].append(block)
	
def is_block_at(position):
	return position in position_to_blocklist and len(position_to_blocklist[position]) > 0

def get_block_types_at(position):
	for block in get_blocks_at(position):
		yield block.block_type
		
def get_blocks_at(position):
	if position in position_to_blocklist:
		for block in position_to_blocklist[position]:
			yield block
			
def is_block_type_at(position, block_type):
	return block_type in get_block_types_at(position)




########## Player

block_player = None

def move_player(block, delta):
	def is_obstacle_type(block_type):
		return block_type in (BlockType.BUG, BlockType.ROCK, BlockType.TREE, BlockType.FILLER)
	
	def is_obstacle_at(position):
		return len([x for x in get_block_types_at(position) if is_obstacle_type(x)]) > 0
	
	def on_fail():
		bubble = Block(block.position, tile_speech, BlockType.SPEECH)
		question = Block(block.position, tile_question, BlockType.SPEECH)
		
		def remove(event):
			if event.type == AppEvent.RESTART:
				return
			bubble.remove()
			question.remove()
		on_event_once(lambda a: a.type == AppEvent.INPUT_DIRECTION or a.type == AppEvent.RESTART, remove)
	
	dest = sum3d(block.position, delta)
	above = sum3d(dest, (0, 1, 0))
	
	if is_obstacle_at(dest):
		if is_obstacle_at(above):
			on_fail()
			return
		for block_type in get_block_types_at(dest):
			if block_type == BlockType.FILLER:
				block.move(sum3d(delta, (0, 1, 0)))
				return
		on_fail()
		return
	else:
		for i in range(1, 5):
			below = sum3d(dest, (0, -i, 0))
			
			for other_block in get_blocks_at(below):
				if other_block.block_type in (BlockType.FILLER, BlockType.ROCK):
					block.move(sum3d(delta, (0, 1 - i, 0)))
					return
				elif other_block.block_type == BlockType.BUG:
					squish_bug(other_block)
					block.move(sum3d(delta, (0, -i, 0)))
					return
				elif is_obstacle_type(other_block.block_type):
					on_fail()
					return
				else:
					continue
		on_fail()
	
def make_player(position):
	global block_player
	block_player = Block(position, tile_player, BlockType.PLAYER)
	
	def center_camera_on_player():
		move_camera_to(sum3d(block_player.position, (0, -1, 0)))
	center_camera_on_player()
	
	def update_tile():
		if is_block_type_at(block_player.position, BlockType.WATER):
			block_player.tile = tile_player_in_water
		else:
			block_player.tile = tile_player
	update_tile()
	
	
	on_event(lambda a: a.type == AppEvent.BLOCK_MOVED and a.block == block_player, lambda a: (
		center_camera_on_player(),
		update_tile()
	))


########## Bug

num_bugs = 0
def restart_bugs():
	global num_bugs
	num_bugs = 0
on_event(lambda a: a.type == AppEvent.RESTART, lambda a: restart_bugs())

def squish_bug(block):
	global num_bugs
	num_bugs -= 1
	
	block.tile = tile_bug_dead
	block.block_type = BlockType.BUG_DEAD
	
	proc_event(pygame.event.Event(AppEvent.BUG_SQUISHED, block=block))

def can_bug_move(block, delta):
	if len(list(get_blocks_at(sum3d(block.position, (0, 1, 0))))) > 0: # something on top, usually another bug
		return False # let other bug move off first so odd behavior doesn't commence
	
	def is_obstacle(block_type):
		return block_type in (BlockType.PLAYER, BlockType.BUG, BlockType.ROCK, BlockType.TREE, BlockType.FILLER)
	def is_obstacle_at(position):
		return len([x for x in get_block_types_at(position) if is_obstacle(x)]) > 0
	
	dest = sum3d(block.position, delta)
	
	if is_obstacle_at(dest):
		return False
	else:
		below = sum3d(dest, (0, -1, 0))
		
		for other_block in get_blocks_at(below):
			if other_block.block_type in (BlockType.FILLER, BlockType.ROCK, BlockType.BUG):
				return True
			elif not is_obstacle(other_block.block_type):
				continue
			return False
		
def move_bug(block, delta):
	if can_bug_move(block, delta):
		block.move(delta)
		
def make_bug(position):
	global num_bugs
	num_bugs += 1
	
	last_move = None
	def do_ai(event):
		nonlocal last_move
		
		if block.block_type == BlockType.BUG_DEAD: return
		
		deltas = [(1, 0, 0), (-1, 0, 0), (0, 0, 1), (0, 0, -1)]
		
		reverse_move = (None if last_move == None else negate3d(last_move))
		if last_move != None:
			deltas.remove(reverse_move) ### avoid about-face when possible 
		
		options = [x for x in deltas if can_bug_move(block, x)]
		if len(options) > 0:
			option = random.choice(options)
			move_bug(block, option)
			last_move = option
		else:
			### attempt about-face when all else fails
			if reverse_move != None and can_bug_move(block, reverse_move):
				move_bug(block, reverse_move)
				last_move = reverse_move
	
	block = Block(position, tile_bug, BlockType.BUG)
	on_event(lambda a: a.type == AppEvent.BLOCK_MOVED and a.block == block_player, do_ai)
	### TODO: Remove event binding when bug-death event fires
	



def make_grass(position):
	Block(position, tile_grass, BlockType.FILLER)
def make_stone(position):
	Block(position, tile_gray, BlockType.FILLER)
def make_boulder(position):
	Block(position, tile_rock, BlockType.ROCK)
def make_tree(position):
	Block(position, tile_tree, BlockType.TREE)
def make_bush_0(position):
	Block(position, tile_bush_0, BlockType.TREE)
def make_bush_1(position):
	Block(position, tile_bush_1, BlockType.TREE)
def make_wood_floor(position):
	Block(position, tile_wood, BlockType.FILLER)
def make_water(position):
	Block(position, tile_water, BlockType.WATER)
def make_dirt(position):
	Block(position, tile_dirt, BlockType.FILLER)


scale_cache = {} ### { surface: ((x, y), result_surface) }
def scale_surface_to(surface, dest_size):
	if not (surface in scale_cache and scale_cache[surface][0] == dest_size):
		scale_cache[surface] = (dest_size, pygame.transform.smoothscale(surface, dest_size))
	return scale_cache[surface][1]

def draw_tile(tile, position):
	scale_world2screen = surface_screen.get_height() / (camera[1] * TILE_L)
	scale_image2tile = tile.surface.get_width() / tile.reg_width
	
	dest_w = scale_world2screen * scale_image2tile
	dest_h = dest_w * tile.surface.get_height() / tile.surface.get_width()
	surface_screen.blit(
		scale_surface_to(tile.surface, int2d((dest_w, dest_h))),
		sum2d(
			### positioning before camera motion
			dif2d(
				scale2d(
					(
						position[0] * TILE_W,
						position[2] * TILE_L - position[1] * TILE_H
					),
					scale_world2screen
				),
				scale2d(tile.reg_position, scale_world2screen / tile.reg_width)
			),
			
			sum2d(
				### camera position
				scale2d(
					(TILE_W * -camera[0][0], TILE_H * camera[0][1] - TILE_L * camera[0][2]), 
					scale_world2screen
				),
				(surface_screen.get_width() / 2, surface_screen.get_height() / 2)
			)
		)
	)

def draw_environment(event):
	def is_filler_at(position):
		for block in get_blocks_at(position):
			if block.tile.is_filler:
				return True
		return False
	
	for block in blocks:
		draw_tile(block.tile, block.position)
		if block.tile.is_filler:
			### surface shadows
			if not is_filler_at(sum3d(block.position, (0, 1, 0))):
				if is_filler_at(sum3d(block.position, (1, 1, 0))):
					draw_tile(shadow_e, block.position)
				if is_filler_at(sum3d(block.position, (-1, 1, 0))):
					draw_tile(shadow_w, block.position)
				if is_filler_at(sum3d(block.position, (0, 1, 1))):
					draw_tile(shadow_s, block.position)
				if is_filler_at(sum3d(block.position, (0, 1, -1))):
					draw_tile(shadow_n, block.position)
				
				### corners
				if not is_filler_at(sum3d(block.position, (0, 1, 1))):
					if is_filler_at(sum3d(block.position, (1, 1, 1))) and not is_filler_at(sum3d(block.position, (1, 1, 0))):
						draw_tile(shadow_se, block.position)
					if is_filler_at(sum3d(block.position, (-1, 1, 1))) and not is_filler_at(sum3d(block.position, (-1, 1, 0))):
						draw_tile(shadow_sw, block.position)
				if is_filler_at(sum3d(block.position, (-1, 1, -1))) and not is_filler_at(sum3d(block.position, (0, 1, -1))) and not is_filler_at(sum3d(block.position, (-1, 1, 0))):
					draw_tile(shadow_nw, block.position)
				if is_filler_at(sum3d(block.position, (1, 1, -1))) and not is_filler_at(sum3d(block.position, (0, 1, -1))) and not is_filler_at(sum3d(block.position, (1, 1, 0))):
					draw_tile(shadow_ne, block.position)
			
			### front shadows
			if not is_filler_at(sum3d(block.position, (0, 0, 1))):
				if is_filler_at(sum3d(block.position, (-1, 0, 1))):
					draw_tile(shadow_side_w, block.position)
					
on_event(lambda a: a.type == EVENT_DRAW, draw_environment)

on_event(lambda a: a.type == AppEvent.INPUT_DIRECTION and a.direction == Direction.NORTH, lambda a: move_player(block_player, (0, 0, -1)))
on_event(lambda a: a.type == AppEvent.INPUT_DIRECTION and a.direction == Direction.SOUTH, lambda a: move_player(block_player, (0, 0, 1)))
on_event(lambda a: a.type == AppEvent.INPUT_DIRECTION and a.direction == Direction.EAST, lambda a: move_player(block_player, (1, 0, 0)))
on_event(lambda a: a.type == AppEvent.INPUT_DIRECTION and a.direction == Direction.WEST, lambda a: move_player(block_player, (-1, 0, 0)))

def ms_to_hms_string(ms):
	seconds = int(ms / 1000)
	
	minutes = int(seconds / 60)
	seconds %= 60
	
	hours = int(minutes / 60)
	minutes %= 60
	
	return ('{0:02d}'.format(hours) + ':' if hours > 0 else '') + '{0:02d}'.format(minutes) + ':' + '{0:02d}'.format(seconds)

fps_clock = pygame.time.Clock()
gui_font_large = pygame.font.SysFont('Verdana', 32)
gui_font_medium = pygame.font.SysFont('Verdana', 20)
gui_font_small = pygame.font.SysFont('Verdana', 14)

puzzle_start = pygame.time.get_ticks()
puzzle_end = None

def restart_puzzle_time():
	global puzzle_start
	global puzzle_end
	puzzle_start = pygame.time.get_ticks()
	puzzle_end = None
on_event(lambda a: a.type == AppEvent.RESTART, lambda a: restart_puzzle_time())

def get_puzzle_duration():
	if puzzle_end != None:
		return puzzle_end - puzzle_start
	return pygame.time.get_ticks() - puzzle_start

def end_puzzle():
	global puzzle_end
	puzzle_end = pygame.time.get_ticks()
on_event(lambda a: a.type == AppEvent.BUG_SQUISHED and num_bugs == 0, lambda a: end_puzzle())

def draw_hud(event):
	fps_clock.tick()
	
	### Keep text away from edges so it's inside the title safe zone
	render_text('FPS: ' + str(int(fps_clock.get_fps())), sum2d((45, -45), surface_screen.get_rect().bottomleft), (0, 1), surface_screen, gui_font_small, color_HUD, color_HUD_outline)
	
	instruction_rect_0 = render_text('Squish all the bugs.', sum2d((-45, 45), surface_screen.get_rect().topright), (1, 0), surface_screen, gui_font_medium, color_HUD, color_HUD_outline)
	instruction_rect_1 = render_text('Use the arrow keys or d-pad to move.', sum2d((0, 10), instruction_rect_0.bottomright), (1, 0), surface_screen, gui_font_medium, color_HUD, color_HUD_outline)
	instruction_rect_2 = render_text('If the puzzle becomes unsolvable, press', instruction_rect_1.bottomright, (1, 0), surface_screen, gui_font_medium, color_HUD, color_HUD_outline)
	render_text('Space or any gamepad button to start over.', instruction_rect_2.bottomright, (1, 0), surface_screen, gui_font_medium, color_HUD, color_HUD_outline)
	
	if num_bugs > 0:
		render_text('Elapsed: ' + ms_to_hms_string(get_puzzle_duration()), (45, 45), (0, 0), surface_screen, gui_font_large, color_HUD, color_HUD_outline)
	else:
		render_text('Solved in: ' + ms_to_hms_string(get_puzzle_duration()), (45, 45), (0, 0), surface_screen, gui_font_large, color_HUD, color_HUD_outline)
on_event(lambda a: a.type == EVENT_DRAW, draw_hud)



######################################################################### begin

### obviously not the best map loading solution in a large app, but whatever
import map_0

def load_map():
	clear_blocks()
	map_0.make(make_player, make_bug, make_grass, make_stone, make_boulder, make_tree, make_bush_0, make_bush_1, make_wood_floor, make_water, make_dirt)
load_map()

on_event(lambda a: a.type == AppEvent.RESTART, lambda a: load_map())

on_event(
	lambda a: (a.type == pygame.KEYDOWN and a.key == pygame.K_SPACE) or a.type == pygame.JOYBUTTONDOWN,
	lambda a: proc_event(pygame.event.Event(AppEvent.RESTART))
)


event_loop()
